class LiftedObservableResource {
    constructor(observable, disposable) {
        this.observable = observable;
        this.disposable = disposable;
    }
    get isDisposed() {
        return this.disposable.isDisposed;
    }
    add(disposable, ...disposables) {
        this.disposable.add(disposable, ...disposables);
        return this;
    }
    dispose() {
        this.disposable.dispose();
    }
    remove(disposable, ...disposables) {
        this.disposable.remove(disposable, ...disposables);
        return this;
    }
    subscribe(subscriber) {
        this.observable.subscribe(subscriber);
    }
}
export const lift = (operator) => observableResource => {
    const liftedObservable = operator(observableResource instanceof LiftedObservableResource
        ? observableResource.observable
        : observableResource);
    const disposable = observableResource instanceof LiftedObservableResource
        ? observableResource.disposable
        : observableResource;
    return new LiftedObservableResource(liftedObservable, disposable);
};
//# sourceMappingURL=index.js.map